from __future__ import absolute_import

try:
    from io import BytesIO
except ImportError:
    from StringIO import StringIO as BytesIO

import qrcode
from reportlab.platypus.flowables import Image

from ..styles import styles


def qrcode_image(data):
    # create QRCode object
    code = qrcode.QRCode(
            box_size=1,
            border=0,
            error_correction=qrcode.constants.ERROR_CORRECT_M)
    code.add_data(data)
    code.make()
    # render QRCode as PNG into memory
    img = code.make_image()
    img_data = BytesIO()
    img.save(img_data, 'PNG')
    img_data.seek(0)
    img_data.__repr__ = lambda: 'qrcode.png'
    # create image floatable
    return Image(img_data)


def sepa_credit_transfer(account_holder, iban, bic, amount, reference,
        purpose='', currency='EUR'):
    '''
    Create QRCode object according to EPC069-12:
    http://www.europeanpaymentscouncil.eu/knowledge_bank_detail.cfm?documents_id=607
    A list of purpose options can be found online:
    http://www.hettwer-beratung.de/sepa-spezialwissen/sepa-technische-anforderungen/sepa-purpose-codes-vs-dta-textschl%C3%BCssel/
    '''
    assert 1 <= len(account_holder) < 70
    assert len(currency) == 3
    assert len(bic) in (8, 11)
    assert ' ' not in bic
    assert 1 <= len(iban) <= 34
    assert ' ' not in iban
    assert len(purpose) <= 4
    assert len(reference) <= 35
    return qrcode_image('\n'.join((
        # Service Tag
        'BCD',
        # Version
        '001',
        # Character set (1=utf-8)
        '1',
        # identification code
        'SCT',
        # BIC
        bic,
        # name
        account_holder,
        # IBAN
        iban,
        # Amount
        '%s%.2f' % (currency, amount),
        # Purpose
        purpose,
        # Reference
        reference,
        # Unstructured Remittance Information
        '',
        # Beneficiary to originator information
        '')))



# -*- coding: utf-8 -*-

from decimal import Decimal
from xml.sax.saxutils import escape

from reportlab.lib import colors
from reportlab.lib.units import mm, cm
from reportlab.platypus import Paragraph
from reportlab.platypus import Spacer
from reportlab.platypus.tables import Table
from reportlab.platypus.tables import TableStyle

from ..constants import CONTENT_WIDTH
from ..optional_django import ugettext as _
from ..optional_django import number_format
from ..styles import styles


def BankTransferForm(account_holder, iban, bic, reference, amount,
        currency=u'EUR', currency_label=u'€', show_qrcode=False):

    col_widths = [40*mm, 60*mm]
    row_heights = [7*mm, 6*mm, 6*mm, 6*mm, 7*mm]

    table_style = []

    if show_qrcode:
        from ..contrib import qrcode
        table_style += [
            ('SPAN', (2, 0), (2, 4)),
        ]
        qrcode_size = sum(row_heights) - 4*mm
        col_widths += [qrcode_size + 4*mm]
        qrcode_image = qrcode.sepa_credit_transfer(
            account_holder=account_holder,
            iban=''.join(iban.split()),
            bic=''.join(bic.split()),
            reference=reference,
            amount=amount,
            currency=currency)
        qrcode_image.drawWidth = qrcode_size
        qrcode_image.drawHeight = qrcode_size

    table_style += [
        ('ALIGN', (0, 0), (-1, -1), 'LEFT'),
        ('VALIGN', (0, 0), (-1, -1), 'TOP'),
        ('TOPPADDING', (0, 0), (-1, -1), 1*mm),
        ('RIGHTPADDING', (0, 0), (-1, -1), 2*mm),
        ('BOTTOMPADDING', (0, 0), (-1, -1), 1*mm),
        ('LEFTPADDING', (0, 0), (-1, -1), 2*mm),
        ('NOSPLIT', (0, 0), (-1, -1)),
        # right
        ('RIGHTPADDING', (-1, 0), (-1, -1), 2*mm),
        ('LINEAFTER', (-1, 0), (-1, -1), 0.2*mm, colors.black),
        # left
        ('LEFTPADDING', (0, 0), (0, -1), 2*mm),
        ('LINEBEFORE', (0, 0), (0, -1), 0.2*mm, colors.black),
        # top
        ('TOPPADDING', (0, 0), (-1, 0), 2*mm),
        ('LINEABOVE', (0, 0), (-1, 0), 0.2*mm, colors.black),
        # bottom
        ('BOTTOMPADDING', (0, -1), (-1, -1), 3*mm),
        ('LINEBELOW', (0, -1), (-1, -1), 0.2*mm, colors.black),
    ]

    def data_generator():
        yield (
            Paragraph(_(u'Account holder') + ':', styles['TableCell']),
            Paragraph(account_holder, styles['TableCell'])
        ) + ((qrcode_image,) if show_qrcode else ())
        yield (
            Paragraph(_(u'IBAN') + ':', styles['TableCell']),
            Paragraph(iban, styles['TableCell']),
        )
        yield (
            Paragraph(_(u'BIC') + ':', styles['TableCell']),
            Paragraph(bic, styles['TableCell']),
        )
        yield (
            Paragraph(_(u'Amount') + ':', styles['TableCell']),
            Paragraph(u'%s %s' % (number_format(amount, 2), currency_label), styles['TableCell']),
        )
        yield (
            Paragraph(_(u'Reference') + ':', styles['TableCell']),
            Paragraph(reference, styles['TableCell']),
        )

    return Table(
        data=list(data_generator()),
        colWidths=col_widths,
        rowHeights=row_heights,
        style=TableStyle(table_style),
        hAlign='LEFT')



# -*- coding: utf-8 -*-

from decimal import Decimal

from .vat_item import VatItem


class Invoice(object):

    def __init__(self, items=None, currency=u'€'):
        self.items = items or []
        self.currency = currency
        self.vat_items = []
        self.recalculate()

    def recalculate(self):
        self.vat_items = []
        d = {}
        for item in self.items:
            if not item.vat_rate:
                continue
            try:
                vat_item = d[item.vat_rate]
            except KeyError:
                vat_item = VatItem(rate=item.vat_rate)
                d[item.vat_rate] = vat_item
            vat_item.amount += item.vat_rate * item.subtotal
        self.vat_items = sorted(d.values(), key=lambda item: item.rate)

    @property
    def gross(self):
        return self.net + sum(vat_item.amount for vat_item in self.vat_items)

    @property
    def net(self):
        return sum(item.total for item in self.items)



# -*- coding: utf-8 -*-

from xml.sax.saxutils import escape

from reportlab import platypus
from reportlab.lib import colors
from reportlab.lib.units import mm, cm
from reportlab.platypus import Frame
from reportlab.platypus import PageTemplate
from reportlab.platypus import Paragraph
from reportlab.platypus import KeepInFrame
from reportlab.platypus import Table
from reportlab.platypus import TableStyle

from .constants import PAGE_SIZE, PAGE_WIDTH, PAGE_HEIGHT
from .constants import CONTENT_LEFT, CONTENT_RIGHT, CONTENT_WIDTH
from .styles import styles


# Address according to DIN 676 und DIN 5008
ADDRESS_WIDTH = 85*mm
ADDRESS_HEIGHT = 55*mm
ADDRESS_X = CONTENT_LEFT # correct would be 20mm, but this looks very strange
ADDRESS_Y = PAGE_HEIGHT - ADDRESS_HEIGHT - 45*mm - 3*mm

SENDER_X = ADDRESS_X
SENDER_Y = ADDRESS_Y + ADDRESS_HEIGHT - 13*mm
SENDER_HEIGHT = 10*mm
SENDER_WIDTH = ADDRESS_WIDTH

SENDER_LINE_Y = PAGE_HEIGHT - 55*mm
SENDER_LINE_COLOR = colors.black
SENDER_LINE_WIDTH = 0.1*mm

RECIPIENT_X = ADDRESS_X
RECIPIENT_Y = ADDRESS_Y
RECIPIENT_HEIGHT = ADDRESS_HEIGHT - 10*mm
RECIPIENT_WIDTH = PAGE_WIDTH - ADDRESS_X

INFOBOX_WIDTH = 75*mm
INFOBOX_HEIGHT = 100*mm
INFOBOX_X = PAGE_WIDTH - INFOBOX_WIDTH - CONTENT_RIGHT
INFOBOX_Y = PAGE_HEIGHT - INFOBOX_HEIGHT - 20*mm

DATE_Y = 45*mm
DATE_HEIGHT = PAGE_HEIGHT-140*mm


class BasePageTemplate(PageTemplate, object):

    def __init__(self, document, *args, **kwargs):
        self.document = document
        super(BasePageTemplate, self).__init__(
            *args, **kwargs)

    def afterDrawPage(self, canvas, document):
        self.draw_header(canvas)
        self.draw_footer(canvas)
        self.draw_marks(canvas)

    def draw_header(self, canvas):
        pass

    def draw_marks(self, canvas):
        canvas.saveState()
        canvas.setLineWidth(0.1*mm)

        # upper fold mark
        p = canvas.beginPath()
        p.moveTo(4*mm, PAGE_HEIGHT-105*mm)
        p.lineTo(8*mm, PAGE_HEIGHT-105*mm)
        canvas.drawPath(p)

        # lower fold mark
        p = canvas.beginPath()
        p.moveTo(4*mm, PAGE_HEIGHT-210*mm)
        p.lineTo(8*mm, PAGE_HEIGHT-210*mm)
        canvas.drawPath(p)

        # center mark
        p = canvas.beginPath()
        p.moveTo(6*mm, PAGE_HEIGHT / 2)
        p.lineTo(13*mm, PAGE_HEIGHT / 2)
        canvas.drawPath(p)

        canvas.restoreState()

    def draw_footer(self, canvas):
        pass

    
class FirstPageTemplate(BasePageTemplate):

    def __init__(self, document):
        super(FirstPageTemplate, self).__init__(
            document=document,
            id='First', frames=[
                # left=25mm, right=10mm, top=115mm, bottom=15mm
                Frame(
                    x1=CONTENT_LEFT, y1=45*mm,
                    width=CONTENT_WIDTH, height=PAGE_HEIGHT-140*mm,
                    leftPadding=0, bottomPadding=0, rightPadding=0, topPadding=0)
            ])

    def draw_address(self, canvas):

        '''
        canvas.saveState()

        # sender line
        canvas.setStrokeColor(SENDER_LINE_COLOR)
        canvas.setLineWidth(SENDER_LINE_WIDTH)
        p = canvas.beginPath()
        p.moveTo(0, SENDER_LINE_Y)
        p.lineTo(PAGE_WIDTH, SENDER_LINE_Y)
        canvas.drawPath(p)

        canvas.restoreState()
        '''

        # sender text
        sender = Frame(
                SENDER_X, SENDER_Y,
                SENDER_WIDTH, SENDER_HEIGHT,
                0, 0, 0, 0)
        sender.add(
                Paragraph(
                    u' · '.join(map(escape, self.document.sender)),
                    styles['Sender']),
                canvas)

        # recipient text
        recipient = Frame(
                RECIPIENT_X, RECIPIENT_Y,
                RECIPIENT_WIDTH, RECIPIENT_HEIGHT,
                0, 0, 0, 0)
        recipient.add(
                Paragraph(
                    u'<br/>'.join(map(escape, self.document.recipient)),
                    styles['Recipient']),
                canvas)

    def draw_infobox(self, canvas):
        infobox = Frame(
                INFOBOX_X, INFOBOX_Y,
                INFOBOX_WIDTH, INFOBOX_HEIGHT,
                0, 0, 0, 0)
        for floatable in self.document.infobox:
            infobox.add(floatable, canvas)

    def draw_date(self, canvas):
        frame = Frame(
                CONTENT_LEFT, DATE_Y,
                CONTENT_WIDTH, DATE_HEIGHT,
                0, 0, 0, 0)
        frame.add(
                Paragraph(
                    escape(self.document.date),
                    styles['Date']),
                canvas)

    def afterDrawPage(self, canvas, document):
        BasePageTemplate.afterDrawPage(self, canvas, document)
        self.draw_address(canvas)
        self.draw_infobox(canvas)
        self.draw_date(canvas)


class LaterPageTemplate(BasePageTemplate):

    def __init__(self, document):
        super(LaterPageTemplate, self).__init__(
                document=document,
                id='Later', frames=[
                    # left=20mm, right=10mm, top=20mm, bottom=15mm
                    Frame(25*mm, 40*mm, PAGE_WIDTH-35*mm, PAGE_HEIGHT-90*mm)
                ])


class BriefTemplate(platypus.BaseDocTemplate):

    def __init__(self, fh, document):
        # super can not be used as BaseDocTemplate is an old style class.
        platypus.BaseDocTemplate.__init__(self,
            fh,
            pagesize=PAGE_SIZE,
            pageTemplates=[
                FirstPageTemplate(document),
                LaterPageTemplate(document)
            ],
            title=document.title,
            subject=document.subject,
            author=document.author,
            keywords=document.keywords,
            creator=document.creator
        )
    
    def handle_pageBegin(self):
        self._handle_pageBegin()
        self._handle_nextPageTemplate('Later')



from decimal import Decimal


class Item(object):

    def __init__(self, position=0, text='', period='', date=None,
            price=Decimal(0), unit='', quantity=Decimal(1),
            discount=Decimal(0), vat_rate=Decimal(0)):
        self.position = position
        self.text = text
        self.period = period
        self.date = date
        self.price = price
        self.unit = unit
        self.quantity = quantity
        self.discount = discount
        self.vat_rate = vat_rate

    @property
    def subtotal(self):
        return self.price * self.quantity

    @property
    def discount_percentage(self):
        return self.discount * 100

    @property
    def discount_amount(self):
        return self.discount * self.subtotal

    @property
    def total(self):
        return self.subtotal - self.discount_amount



# -*- coding: utf-8 -*-

from decimal import Decimal
from xml.sax.saxutils import escape

from reportlab.lib import colors
from reportlab.lib.units import mm, cm
from reportlab.platypus import Paragraph
from reportlab.platypus import Spacer
from reportlab.platypus.tables import Table
from reportlab.platypus.tables import TableStyle

from ..constants import CONTENT_WIDTH
from ..optional_django import ugettext as _
from ..optional_django import number_format
from ..styles import styles


def TotalTable(invoice):

    table_style = [
        ('ALIGN', (0, 0), (-1, -1), 'LEFT'),
        ('TOPPADDING', (0, 0), (-1, -1), 1*mm),
        ('RIGHTPADDING', (0, 0), (-1, -1), 2*mm),
        ('BOTTOMPADDING', (0, 0), (-1, -1), 1*mm),
        ('LEFTPADDING', (0, 0), (-1, -1), 2*mm),
    ]

    col_widths = [0, 32*mm, 24*mm]
    col_widths[0] = CONTENT_WIDTH - sum(col_widths)

    if invoice.vat_items:
        net_row = 0
        vat_row = net_row+1
        gross_row = vat_row + len(invoice.vat_items)
        table_style += [
            ('BOTTOMPADDING', (0, gross_row-1), (-1, gross_row-1), 2*mm),
            #('TOPPADDING', (0, vat_row), (-1, gross_row), 1*mm),
            #('BOTTOMPADDING', (0, vat_row), (-1, gross_row), 1*mm),
            ('LINEABOVE', (0, net_row), (-1, net_row), 0.3*mm, colors.black),
            ('LINEABOVE', (1, gross_row), (-1, gross_row), 0.3*mm, colors.black),
        ]
        def data_generator():
            yield (
                Paragraph(u'', styles['TableCell']),
                Paragraph(_(u'Sum (net)'), styles['TableCell']),
                Paragraph(u'%s €' % number_format(invoice.net, 2),
                    styles['TableNumber']),
            )
            for vat_item in invoice.vat_items:
                yield (
                    Paragraph(u'', styles['TableCell']),
                    Paragraph((u'+%s%% ' % number_format(vat_item.rate * 100)) +
                        _('VAT'), styles['TableCell']),
                    Paragraph(u'%s €' % number_format(vat_item.amount, 2),
                        styles['TableNumber']),
                )
            yield (
                Paragraph(u'', styles['TableCell']),
                Paragraph(_(u'Sum (gross)'), styles['GrossTableCell']),
                Paragraph(u'%s €' % number_format(invoice.gross, 2),
                    styles['GrossValueTableCell']),
            )
    else:
        net_row = 0
        gross_row = 1
        table_style += [
            ('TOPPADDING', (0, net_row), (-1, net_row), 2*mm),
            ('BOTTOMPADDING', (0, net_row), (-1, net_row), 0),
            ('LINEABOVE', (0, net_row), (-1, net_row), 0.3*mm, colors.black),
            ('TOPPADDING', (0, gross_row), (-1, gross_row), 2*mm),
            ('LINEABOVE', (1, gross_row), (-1, gross_row), 0.3*mm, colors.black),
        ]
        def data_generator():
            yield (
                Spacer(0, 0),
                Spacer(0, 0),
                Spacer(0, 0),
            )
            yield (
                Paragraph(u'', styles['TableCell']),
                Paragraph(_(u'Sum (net)'), styles['GrossTableCell']),
                Paragraph(u'%s €' % number_format(invoice.gross, 2),
                    styles['GrossValueTableCell']),
            )

    return Table(
        data=list(data_generator()),
        colWidths=col_widths,
        style=TableStyle(table_style))



# Use functions provided by django but do not depend on it.

try:
    from django.utils.translation import gettext, ugettext
except ImportError:
    from gettext import gettext
    ugettext = gettext

try:
    from django.utils.formats import number_format
except ImportError:
    def number_format(value, decimal_places=''):
        format = '%%.%sf' % decimal_places
        return format % value

try:
    from django.utils.formats import date_format
except ImportError:
    def date_format(value, format=None):
        return u'%s' % value



# -*- coding: utf-8 -*-

from decimal import Decimal
from xml.sax.saxutils import escape

from reportlab.lib import colors
from reportlab.lib.units import mm, cm
from reportlab.platypus import Paragraph
from reportlab.platypus import Spacer
from reportlab.platypus.tables import Table
from reportlab.platypus.tables import TableStyle

from ..optional_django import ugettext as _
from ..styles import styles


class FlowableProxy(object):

    def __getattr__(self, name):
        return getattr(self.flowable, name)


class Field(FlowableProxy):
    field_height = 5*mm
    def __init__(self, label):
        self.flowable = Table(
            data=[
                [''], # empty row for the text
                [Paragraph(label, styles['FieldLabel'])],
            ],
            colWidths=[80*mm],
            rowHeights=[self.field_height, None],
            style=TableStyle([
                ('NOSPLIT', (0, 0), (-1, -1)),
                ('TOPPADDING', (0, 0), (-1, -1), 0),
                ('RIGHTPADDING', (0, 0), (-1, -1), 0),
                ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
                ('LEFTPADDING', (0, 0), (-1, -1), 0),
                ('LINEBELOW', (0, 0), (0, 0), 0.2*mm, colors.black),
                #('TOPPADDING', (0, 0), (0, 0), 5*mm),
            ]),
            hAlign='LEFT'
        )


class SignatureField(Field):
    field_height = 15*mm

    def __init__(self, label=None):
        if label is None:
            label = _('Place, date and signature')
        super(SignatureField, self).__init__(label)


class TwoSignaturesField(Field):
    field_height = 15*mm

    def __init__(self, label_left, label_right, field_height=None):
        if field_height is not None:
            self.field_height = field_height
        self.flowable = Table(
            data=[
                ['', '', ''], # empty row for the text
                [
                    Paragraph(label_left, styles['FieldLabel']),
                    '',
                    Paragraph(label_right, styles['FieldLabel']),
                ],
            ],
            colWidths=[80*mm, 10*mm, 80*mm],
            rowHeights=[self.field_height, None],
            style=TableStyle([
                ('NOSPLIT', (0, 0), (-1, -1)),
                ('TOPPADDING', (0, 0), (-1, -1), 0),
                ('RIGHTPADDING', (0, 0), (-1, -1), 0),
                ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
                ('LEFTPADDING', (0, 0), (-1, -1), 0),
                ('LINEBELOW', (0, 0), (0, 0), 0.2*mm, colors.black),
                ('LINEBELOW', (2, 0), (2, 0), 0.2*mm, colors.black),
            ]),
            hAlign='LEFT'
        )


class PostalCodeAndCityField(FlowableProxy):
    field_height = 5*mm
    def __init__(self):
        self.flowable = Table(
            data=[
                ['', '', ''], # empty row for the text
                [
                    Paragraph(_('Postal code'), styles['FieldLabel']),
                    '', # Space between postal code and city
                    Paragraph(_('City'), styles['FieldLabel']),
                ],
            ],
            colWidths=[20*mm, 5*mm, 55*mm],
            rowHeights=[self.field_height, None],
            style=TableStyle([
                ('NOSPLIT', (0, 0), (-1, -1)),
                ('TOPPADDING', (0, 0), (-1, -1), 0),
                ('RIGHTPADDING', (0, 0), (-1, -1), 0),
                ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
                ('LEFTPADDING', (0, 0), (-1, -1), 0),
                ('LINEBELOW', (0, 0), (0, 0), 0.2*mm, colors.black),
                ('LINEBELOW', (2, 0), (2, 0), 0.2*mm, colors.black),
            ]),
            hAlign='LEFT'
        )



from reportlab.lib import colors
from reportlab.lib.styles import ParagraphStyle
from reportlab.lib.styles import StyleSheet1
from reportlab.lib.units import mm, cm
from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER


styles = StyleSheet1()

styles.add(ParagraphStyle(
    name='Normal',
    fontName='Helvetica',
    fontSize=10,
    leading=12
))

styles.add(ParagraphStyle(
    name='InfoboxH1',
    parent=styles['Normal'],
    fontSize=16,
    leading=8,
    spaceAfter=10*mm))

styles.add(ParagraphStyle(
    name='InfoboxTitle',
    parent=styles['Normal'],
    fontSize=8,
    leading=8,
    spaceAfter=0.5*mm))

styles.add(ParagraphStyle(
    name='InfoboxText',
    parent=styles['Normal'],
    fontSize=10,
    leading=12,
    spaceAfter=2*mm))

styles.add(ParagraphStyle(
    name='InfoboxNotice',
    parent=styles['Normal'],
    fontName='Helvetica-Bold',
    fontSize=10,
    spaceBefore=4*mm,
    spaceAfter=4*mm))

styles.add(ParagraphStyle(
    name='Footer',
    fontSize=8,
    leading=9,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='FooterTitle',
    fontSize=7,
    leading=10,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='FooterText',
    fontSize=9,
    leading=10,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='Sender',
    fontSize=8,
    leading=8,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='Recipient',
    fontSize=12,
    leading=14,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='Text',
    spaceBefore=2*mm,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='Subject',
    fontSize=12,
    leading=12,
    spaceAfter=8*mm,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='Date',
    alignment=TA_RIGHT,
    parent=styles['Subject']))

styles.add(ParagraphStyle(
    name='Greeting',
    spaceBefore=2*mm,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='Message',
    spaceBefore=2*mm,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='H2',
    fontName='Helvetica-Bold',
    fontSize=10,
    spaceBefore=4*mm,
    spaceAfter=4*mm,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='TableCell',
    fontSize=10,
    leading=10,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='TableHead',
    fontName='Helvetica-Bold',
    parent=styles['TableCell']))

styles.add(ParagraphStyle(
    name='TableHeadRight',
    alignment=TA_RIGHT,
    parent=styles['TableHead']))

styles.add(ParagraphStyle(
    name='TableNumber',
    alignment=TA_RIGHT,
    parent=styles['TableCell']))

styles.add(ParagraphStyle(
    name='GrossTableCell',
    fontName='Helvetica-Bold',
    parent=styles['TableCell']))

styles.add(ParagraphStyle(
    name='GrossValueTableCell',
    alignment=TA_RIGHT,
    parent=styles['GrossTableCell']))

styles.add(ParagraphStyle(
    name='Terms',
    spaceAfter=2*mm,
    parent=styles['Normal']))

styles.add(ParagraphStyle(
    name='Closing',
    spaceBefore=8*mm,
    parent=styles['Text']))

styles.add(ParagraphStyle(
    name='Signature',
    parent=styles['Text']))

styles.add(ParagraphStyle(
    name='FieldLabel',
    parent=styles['Normal'],
    fontSize=8,
    spaceAfter=0))



from functools import partial

from reportlab.lib.units import mm
from reportlab.platypus import Paragraph
from reportlab.platypus import Spacer

from ..styles import styles


P = partial(Paragraph, style=styles['Text'])
H2 = partial(Paragraph, style=styles['H2'])
Subject = partial(Paragraph, style=styles['Subject'])
SmallSpacer = partial(Spacer, width=0, height=5*mm)
MediumSpacer = partial(Spacer, width=0, height=10*mm)
LargeSpacer = partial(Spacer, width=0, height=20*mm)



# -*- coding: utf-8 -*-

from decimal import Decimal
from functools import partial
from xml.sax.saxutils import escape

from reportlab.lib import colors
from reportlab.lib.units import mm, cm
from reportlab.platypus import Paragraph
from reportlab.platypus.tables import Table
from reportlab.platypus.tables import TableStyle

from ..constants import CONTENT_WIDTH
from ..optional_django import ugettext as _
from ..optional_django import number_format
from ..optional_django import date_format
from ..styles import styles


Head = partial(Paragraph, style=styles['TableHead'])
HeadRight = partial(Paragraph, style=styles['TableHeadRight'])
Cell = partial(Paragraph, style=styles['TableCell'])
Number = partial(Paragraph, style=styles['TableNumber'])


def ItemTable(invoice):

    style = [
        ('VALIGN', (0, 0), (-1,  0), 'BOTTOM'),
        ('VALIGN', (0, 1), (-1, -1), 'TOP'),
        ('LINEBELOW', (0, 0), (-1, 0), 0.3*mm, colors.black),
        #('LINEBELOW', (0, 1), (-1, -1), 0.1*mm, colors.black),
        ('TOPPADDING', (0, 0), (-1, -1), 2*mm),
        ('RIGHTPADDING', (0, 0), (-1, -1), 2*mm),
        ('BOTTOMPADDING', (0, 0), (-1, -1), 3*mm),
        ('LEFTPADDING', (0, 0), (-1, -1), 2*mm),
        # no padding between net price and unit
        ('RIGHTPADDING', (3, 0), (3, -1), 0),
        ('LEFTPADDING', (4, 0), (4, -1), 0),
    ]

    show_period_column = any(item.period for item in invoice.items)
    show_date_column = any(item.date for item in invoice.items) and \
            not show_period_column

    col_widths = [
            8*mm, # position
            0,    # description
            46*mm if show_period_column else # period
            24*mm if show_date_column else # date
            0, # neither period nor date
            21*mm, # unit price
            10*mm, # unit
            20*mm, # quantity
            26*mm  # sum price
    ]
    col_widths[1] = CONTENT_WIDTH - sum(col_widths)

    def data_generator():
        # header
        yield (
            HeadRight(u'#'),
            Head(_('Description')),
            Head(_('Period') if show_period_column else
                 _('Date') if show_date_column else ''),
            HeadRight(_('Unit Price')),
            Head(u''),
            HeadRight(_('Quantity')),
            HeadRight(_('Line Total')),
        )
        # items
        row = 0
        for item in invoice.items:
            row += 1
            if not (item.period or item.date):
                style.append(('SPAN', (1, row), (2, row)))
            yield (
                Number(u'%s' % item.position),
                Cell(escape(item.text)),
                Cell(escape(item.period) if item.period else
                     escape(date_format(item.date, 'SHORT_DATE_FORMAT')) if item.date else u''),
                Number(u'%s €' % number_format(item.price, 2)),
                Cell((u'/%s' % escape(item.unit))
                    if item.unit else u''),
                Number(number_format(item.quantity, 2)),
                Number(u'%s €' % number_format(item.subtotal, 2)),
            )
            if item.discount:
                row += 1
                percentage = number_format(item.discount_percentage)
                yield (
                    Cell(u''),
                    Cell((u'%s%% ' % percentage) + _('discount')),
                    Cell(u''),
                    Cell(u''),
                    Cell(u''),
                    Cell(u''),
                    Number(u'–%s €' % number_format(item.discount_amount, 2)),
                )
                style.append(('TOPPADDING', (0, row), (-1, row), 0))
            # draw line below item
            style.append(('LINEBELOW', (0, row), (-1, row),
                0.1*mm, colors.black))

    return Table(
        data=list(data_generator()),
        colWidths=col_widths,
        style=TableStyle(style),
        repeatRows=1)



class Document(object):

    title = ''
    subject = ''
    author = ''
    keywords = None
    creator = 'http://pypi.python.org/pypi/dinbrief'

    sender = None
    recipient = None
    content = None
    date = ''

    def __init__(self, **kwargs):
        # Metadata
        self.title = kwargs.pop('title', self.title)
        self.subject = kwargs.pop('subject', self.subject)
        self.author = kwargs.pop('author', self.author)
        self.keywords = kwargs.pop('keywords', self.keywords)
        self.creator = kwargs.pop('creator', self.creator)
        # Content
        self.sender = kwargs.pop('sender', self.sender or [])
        self.recipient = kwargs.pop('recipient', self.recipient or [])
        self.infobox = kwargs.pop('infobox', [])
        self.date = kwargs.pop('date', self.date)
        self.content = kwargs.pop('content', self.content or [])



#!/usr/bin/env python

import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

# Prevent "TypeError: 'NoneType' object is not callable" when running tests.
# (http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing
except ImportError:
    pass

from dinbrief import __version__ as package_version

setup(
    name='dinbrief',
    version=package_version,
    description='PDF renderer for DIN 5008 and DIN 676 compliant letters and invoices',
    long_description=read('README'),
    author='Michael P. Jung',
    author_email='michael.jung@terreon.de',
    license='BSD',
    keywords='dinbrief DIN 5008 676 brief letter invoice pdf',
    url='https://bitbucket.org/terreon/dinbrief',
    packages=['dinbrief', 'dinbrief.invoice', 'dinbrief.contrib'],
    #test_suite='dinbrief.tests',
    install_requires=read('requirements.txt').split('\n'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from reportlab.lib.pagesizes import A4
from reportlab.lib.units import mm, cm


PAGE_SIZE = A4
PAGE_WIDTH = PAGE_SIZE[0]
PAGE_HEIGHT = PAGE_SIZE[1]

CONTENT_LEFT = 24.1*mm
#CONTENT_RIGHT = 8.1*mm
CONTENT_RIGHT = 13.1*mm
CONTENT_WIDTH = PAGE_WIDTH - CONTENT_LEFT - CONTENT_RIGHT
